@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2020 ZNC, see the NOTICE file for details.
+ * Copyright (C) 2004-2024 ZNC, see the NOTICE file for details.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -136,7 +136,10 @@ class ZNC_EXPORT_LIB_EXPORT CPyModule : public CModule {
                          CString& sMessage) override;
     EModRet OnTopic(CNick& Nick, CChan& Channel, CString& sTopic) override;
     bool OnServerCapAvailable(const CString& sCap) override;
+    bool OnServerCap302Available(const CString& sCap, const CString& sValue) override;
     void OnServerCapResult(const CString& sCap, bool bSuccess) override;
+    void OnClientAttached() override;
+    void OnClientDetached() override;
     EModRet OnTimerAutoJoin(CChan& Channel) override;
     bool OnEmbeddedWebRequest(CWebSock&, const CString&, CTemplate&) override;
     EModRet OnAddNetwork(CIRCNetwork& Network, CString& sErrorRet) override;
@@ -334,3 +337,49 @@ class CModulesIter {
     CModules* m_pModules;
     CModules::const_iterator m_it;
 };
+
+class ZNC_EXPORT_LIB_EXPORT CPyModCommand : public CModCommand {
+  CPyModule* m_pModule;
+  CModPython* m_pModPython;
+  PyObject* m_pyObj;
+
+  void operator()(const CString& sLine);
+
+  public:
+    CPyModCommand(CPyModule* pModule,
+                  const CString& sCmd, const COptionalTranslation& sArgs,
+                  const COptionalTranslation& sDesc, PyObject *pyObj)
+      : CModCommand(sCmd, [=](const CString& sLine) { (*this)(sLine); }, sArgs,
+                    sDesc),
+        m_pModule(pModule),
+        m_pModPython(pModule->GetModPython()),
+        m_pyObj(pyObj) {
+      Py_INCREF(pyObj);
+      pModule->AddCommand(*this);
+    }
+    virtual ~CPyModCommand();
+
+    CPyModule* GetModule();
+};
+
+inline CPyModCommand* CreatePyModCommand(CPyModule* pModule,
+                                         const CString& sCmd,
+                                         const COptionalTranslation& sArgs,
+                                         const COptionalTranslation& sDesc,
+                                         PyObject* pyObj) {
+    return new CPyModCommand(pModule, sCmd, sArgs, sDesc, pyObj);
+}
+
+class ZNC_EXPORT_LIB_EXPORT CPyCapability : public CCapability {
+  public:
+    CPyCapability(PyObject* serverCb, PyObject* clientCb);
+    ~CPyCapability();
+
+    void OnServerChangedSupport(CIRCNetwork* pNetwork, bool bState) override;
+    void OnClientChangedSupport(CClient* pClient, bool bState) override;
+
+  private:
+    PyObject* m_serverCb;
+    PyObject* m_clientCb;
+};
+
