@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2004-2015 ZNC, see the NOTICE file for details.
+# Copyright (C) 2004-2024 ZNC, see the NOTICE file for details.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,11 +14,21 @@
 # limitations under the License.
 #
 
+_cov = None
+import os
+if os.environ.get('ZNC_MODPYTHON_COVERAGE'):
+    import coverage
+    _cov = coverage.Coverage(auto_data=True, branch=True)
+    _cov.start()
+
 from functools import wraps
-import imp
+import collections.abc
+import importlib.abc
+import importlib.machinery
+import importlib.util
 import re
+import sys
 import traceback
-import collections
 
 from znc_core import *
 
@@ -118,7 +128,7 @@ class Timer:
         pass
 
 
-class ModuleNVIter(collections.Iterator):
+class ModuleNVIter(collections.abc.Iterator):
     def __init__(self, cmod):
         self._cmod = cmod
         self.it = cmod.BeginNV_()
@@ -131,7 +141,7 @@ class ModuleNVIter(collections.Iterator):
         return res
 
 
-class ModuleNV(collections.MutableMapping):
+class ModuleNV(collections.abc.MutableMapping):
     def __init__(self, cmod):
         self._cmod = cmod
 
@@ -169,6 +179,28 @@ class Module:
     def __str__(self):
         return self.GetModName()
 
+    @classmethod
+    def t_s(cls, english, context=''):
+        domain = 'znc-' + cls.__name__
+        return CTranslation.Get().Singular(domain, context, english)
+
+    @classmethod
+    def t_f(cls, english, context=''):
+        fmt = cls.t_s(english, context)
+        # Returning bound method
+        return fmt.format
+
+    @classmethod
+    def t_p(cls, english, englishes, num, context=''):
+        domain = 'znc-' + cls.__name__
+        fmt = CTranslation.Get().Plural(domain, context, english, englishes,
+                                        num)
+        return fmt.format
+
+    @classmethod
+    def t_d(cls, english, context=''):
+        return CDelayedTranslation('znc-' + cls.__name__, context, english)
+
     def OnLoad(self, sArgs, sMessage):
         return True
 
@@ -265,7 +297,7 @@ class Module:
         pass
 
     def OnModCommand(self, sCommand):
-        pass
+        self.HandleCommand(sCommand)
 
     def OnModNotice(self, sMessage):
         pass
@@ -342,6 +374,9 @@ class Module:
     def OnUserTopicRequest(self, sChannel):
         pass
 
+    def OnUserQuit(self, sMessage):
+        pass
+
     def OnCTCPReply(self, Nick, sMessage):
         pass
 
@@ -375,6 +410,15 @@ class Module:
     def OnServerCapAvailable(self, sCap):
         pass
 
+    def OnServerCap302Available(self, sCap, sValue):
+        return self.OnServerCapAvailable(sCap)
+
+    def OnClientAttached(self):
+        pass
+
+    def OnClientDetached(self):
+        pass
+
     def OnServerCapResult(self, sCap, bSuccess):
         pass
 
@@ -396,6 +440,16 @@ class Module:
     def OnSendToIRC(self, sLine):
         pass
 
+    # Command stuff
+    def AddCommand(self, cls, *args, **kwargs):
+        cmd = cls(*args, **kwargs)
+        cmd._cmodcommand = CreatePyModCommand(self._cmod, cls.command,
+                                              COptionalTranslation(cls.args),
+                                              COptionalTranslation(cls.description),
+                                              cmd)
+
+        return cmd
+
     # Global modules
     def OnAddUser(self, User, sErrorRet):
         pass
@@ -458,6 +512,198 @@ class Module:
     def OnRawMode2(self, OpNick, Channel, sModes, sArgs):
         return self.OnRawMode(OpNick, Channel, sModes, sArgs)
 
+    def OnRawMessage(self, msg):
+        pass
+
+    def OnNumericMessage(self, msg):
+        pass
+
+    # Deprecated non-Message functions should still work, for now.
+    def OnQuitMessage(self, msg, vChans):
+        return self.OnQuit(msg.GetNick(), msg.GetReason(), vChans)
+
+    def OnNickMessage(self, msg, vChans):
+        return self.OnNick(msg.GetNick(), msg.GetNewNick(), vChans)
+
+    def OnKickMessage(self, msg):
+        return self.OnKick(msg.GetNick(), msg.GetKickedNick(), msg.GetChan(), msg.GetReason())
+
+    def OnJoinMessage(self, msg):
+        return self.OnJoin(msg.GetNick(), msg.GetChan())
+
+    def OnPartMessage(self, msg):
+        return self.OnPart(msg.GetNick(), msg.GetChan(), msg.GetReason())
+
+    def OnChanBufferPlayMessage(self, msg):
+        modified = String()
+        old = modified.s = msg.ToString(CMessage.ExcludeTags)
+        ret = self.OnChanBufferPlayLine(msg.GetChan(), msg.GetClient(), modified)
+        if old != modified.s:
+            msg.Parse(modified.s)
+        return ret
+
+    def OnPrivBufferPlayMessage(self, msg):
+        modified = String()
+        old = modified.s = msg.ToString(CMessage.ExcludeTags)
+        ret = self.OnPrivBufferPlayLine(msg.GetClient(), modified)
+        if old != modified.s:
+            msg.Parse(modified.s)
+        return ret
+
+    def OnUserRawMessage(self, msg):
+        pass
+
+    def OnUserCTCPReplyMessage(self, msg):
+        target = String(msg.GetTarget())
+        text = String(msg.GetText())
+        ret = self.OnUserCTCPReply(target, text)
+        msg.SetTarget(target.s)
+        msg.SetText(text.s)
+        return ret
+
+    def OnUserCTCPMessage(self, msg):
+        target = String(msg.GetTarget())
+        text = String(msg.GetText())
+        ret = self.OnUserCTCP(target, text)
+        msg.SetTarget(target.s)
+        msg.SetText(text.s)
+        return ret
+
+    def OnUserActionMessage(self, msg):
+        target = String(msg.GetTarget())
+        text = String(msg.GetText())
+        ret = self.OnUserAction(target, text)
+        msg.SetTarget(target.s)
+        msg.SetText(text.s)
+        return ret
+
+    def OnUserTextMessage(self, msg):
+        target = String(msg.GetTarget())
+        text = String(msg.GetText())
+        ret = self.OnUserMsg(target, text)
+        msg.SetTarget(target.s)
+        msg.SetText(text.s)
+        return ret
+
+    def OnUserNoticeMessage(self, msg):
+        target = String(msg.GetTarget())
+        text = String(msg.GetText())
+        ret = self.OnUserNotice(target, text)
+        msg.SetTarget(target.s)
+        msg.SetText(text.s)
+        return ret
+
+    def OnUserJoinMessage(self, msg):
+        chan = String(msg.GetTarget())
+        key = String(msg.GetKey())
+        ret = self.OnUserJoin(chan, key)
+        msg.SetTarget(chan.s)
+        msg.SetKey(key.s)
+        return ret
+
+    def OnUserPartMessage(self, msg):
+        chan = String(msg.GetTarget())
+        reason = String(msg.GetReason())
+        ret = self.OnUserPart(chan, reason)
+        msg.SetTarget(chan.s)
+        msg.SetReason(reason.s)
+        return ret
+
+    def OnUserTopicMessage(self, msg):
+        chan = String(msg.GetTarget())
+        topic = String(msg.GetTopic())
+        ret = self.OnUserTopic(chan, topic)
+        msg.SetTarget(chan.s)
+        msg.SetTopic(topic.s)
+        return ret
+
+    def OnUserQuitMessage(self, msg):
+        reason = String(msg.GetReason())
+        ret = self.OnUserQuit(reason)
+        msg.SetReason(reason.s)
+        return ret
+
+    def OnCTCPReplyMessage(self, msg):
+        text = String(msg.GetText())
+        ret = self.OnCTCPReply(msg.GetNick(), text)
+        msg.SetText(text.s)
+        return ret
+
+    def OnPrivCTCPMessage(self, msg):
+        text = String(msg.GetText())
+        ret = self.OnPrivCTCP(msg.GetNick(), text)
+        msg.SetText(text.s)
+        return ret
+
+    def OnChanCTCPMessage(self, msg):
+        text = String(msg.GetText())
+        ret = self.OnChanCTCP(msg.GetNick(), msg.GetChan(), text)
+        msg.SetText(text.s)
+        return ret
+
+    def OnPrivActionMessage(self, msg):
+        text = String(msg.GetText())
+        ret = self.OnPrivAction(msg.GetNick(), text)
+        msg.SetText(text.s)
+        return ret
+
+    def OnChanActionMessage(self, msg):
+        text = String(msg.GetText())
+        ret = self.OnChanAction(msg.GetNick(), msg.GetChan(), text)
+        msg.SetText(text.s)
+        return ret
+
+    def OnPrivTextMessage(self, msg):
+        text = String(msg.GetText())
+        ret = self.OnPrivMsg(msg.GetNick(), text)
+        msg.SetText(text.s)
+        return ret
+
+    def OnChanTextMessage(self, msg):
+        text = String(msg.GetText())
+        ret = self.OnChanMsg(msg.GetNick(), msg.GetChan(), text)
+        msg.SetText(text.s)
+        return ret
+
+    def OnPrivNoticeMessage(self, msg):
+        text = String(msg.GetText())
+        ret = self.OnPrivNotice(msg.GetNick(), text)
+        msg.SetText(text.s)
+        return ret
+
+    def OnChanNoticeMessage(self, msg):
+        text = String(msg.GetText())
+        ret = self.OnChanNotice(msg.GetNick(), msg.GetChan(), text)
+        msg.SetText(text.s)
+        return ret
+
+    def OnTopicMessage(self, msg):
+        topic = String(msg.GetTopic())
+        ret = self.OnTopic(msg.GetNick(), msg.GetChan(), topic)
+        msg.SetTopic(topic.s)
+        return ret
+
+    def OnUnknownUserRawMessage(self, msg):
+        pass
+
+    def OnSendToClientMessage(self, msg):
+        pass
+
+    def OnSendToIRCMessage(self, msg):
+        pass
+
+
+class Command:
+    command = ''
+    args = ''
+    description = ''
+
+    def __call__(self, sLine):
+        pass
+
+    def GetModule(self):
+        return self._cmodcommand.GetModule().GetNewPyObj()
+
 
 def make_inherit(cl, parent, attr):
     def make_caller(parent, name, attr):
@@ -466,57 +712,102 @@ def make_inherit(cl, parent, attr):
         for x in parent.__dict__:
             if not x.startswith('_') and x not in cl.__dict__:
                 setattr(cl, x, make_caller(parent, x, attr))
-        if '_s' in parent.__dict__:
-            parent = parent._s
+        if parent.__bases__:
+            # Multiple inheritance is not supported (yet?)
+            parent = parent.__bases__[0]
         else:
             break
 
 make_inherit(Socket, CPySocket, '_csock')
 make_inherit(Module, CPyModule, '_cmod')
 make_inherit(Timer, CPyTimer, '_ctimer')
+make_inherit(Command, CPyModCommand, '_cmodcommand')
 
 
+class ZNCModuleLoader(importlib.abc.SourceLoader):
+    def __init__(self, modname, pypath):
+        self.pypath = pypath
+
+    def create_module(self, spec):
+        self._datadir = spec.loader_state[0]
+        self._package_dir = spec.loader_state[1]
+        return super().create_module(spec)
+
+    def get_data(self, path):
+        with open(path, 'rb') as f:
+            return f.read()
+
+    def get_filename(self, fullname):
+        return self.pypath
+
+
+class ZNCModuleFinder(importlib.abc.MetaPathFinder):
+    @staticmethod
+    def find_spec(fullname, path, target=None):
+        if fullname == 'znc_modules':
+            spec = importlib.util.spec_from_loader(fullname, None, is_package=True)
+            return spec
+        parts = fullname.split('.')
+        if parts[0] != 'znc_modules':
+            return
+        def dirs():
+            if len(parts) == 2:
+                # common case
+                yield from CModules.GetModDirs()
+            else:
+                # the module is a package and tries to load a submodule of it
+                for libdir in sys.modules['znc_modules.' + parts[1]].__loader__._package_dir:
+                    yield libdir, None
+        for libdir, datadir in dirs():
+            finder = importlib.machinery.FileFinder(libdir,
+                    (ZNCModuleLoader, importlib.machinery.SOURCE_SUFFIXES))
+            spec = finder.find_spec('.'.join(parts[1:]))
+            if spec:
+                spec.name = fullname
+                spec.loader_state = (datadir, spec.submodule_search_locations)
+                # It almost works with original submodule_search_locations,
+                # then python will find submodules of the package itself,
+                # without calling out to ZNCModuleFinder or ZNCModuleLoader.
+                # But updatemod will be flaky for those submodules because as
+                # of py3.8 importlib.invalidate_caches() goes only through
+                # sys.meta_path, but not sys.path_hooks. So we make them load
+                # through ZNCModuleFinder too, but still remember the original
+                # dir so that the whole module comes from a single entry in
+                # CModules.GetModDirs().
+                spec.submodule_search_locations = []
+                return spec
+
+
+sys.meta_path.append(ZNCModuleFinder())
+
+_py_modules = set()
+
 def find_open(modname):
     '''Returns (pymodule, datapath)'''
-    for d in CModules.GetModDirs():
-        # d == (libdir, datadir)
-        try:
-            x = imp.find_module(modname, [d[0]])
-        except ImportError:
-            # no such file in dir d
-            continue
-        # x == (<open file './modules/admin.so', mode 'rb' at 0x7fa2dc748d20>,
-        #       './modules/admin.so', ('.so', 'rb', 3))
-        # x == (<open file './modules/pythontest.py', mode 'U' at
-        #       0x7fa2dc748d20>, './modules/pythontest.py', ('.py', 'U', 1))
-        if x[0] is None and x[2][2] != imp.PKG_DIRECTORY:
-            # the same
-            continue
-        if x[2][0] == '.so':
-            try:
-                pymodule = imp.load_module(modname, *x)
-            except ImportError:
-                # found needed .so but can't load it...
-                # maybe it's normal (non-python) znc module?
-                # another option here could be to "continue"
-                # search of python module in other moddirs.
-                # but... we respect C++ modules ;)
-                return (None, None)
-            finally:
-                x[0].close()
-        else:
-            # this is not .so, so it can be only python module .py or .pyc
-            try:
-                pymodule = imp.load_module(modname, *x)
-            finally:
-                if x[0]:
-                    x[0].close()
-        return (pymodule, d[1]+modname)
+    fullname = 'znc_modules.' + modname
+    for m in _py_modules:
+        if m.GetModName() == modname:
+            break
     else:
-        # nothing found
+        # module is not loaded, clean up previous attempts to load it or even
+        # to list as available modules
+        # This is to to let updatemod work
+        to_remove = []
+        for m in sys.modules:
+            if m == fullname or m.startswith(fullname + '.'):
+                to_remove.append(m)
+        for m in to_remove:
+            del sys.modules[m]
+    try:
+        module = importlib.import_module(fullname)
+    except ImportError:
         return (None, None)
-
-_py_modules = set()
+    if not isinstance(module.__loader__, ZNCModuleLoader):
+        # If modname/ is a directory, it was "loaded" using _NamespaceLoader.
+        # This is the case for e.g. modperl.
+        # https://github.com/znc/znc/issues/1757
+        return (None, None)
+    return (module, os.path.join(module.__loader__._datadir, modname))
 
 def load_module(modname, args, module_type, user, network, retmsg, modpython):
     '''Returns 0 if not found, 1 on loading error, 2 on success'''
@@ -538,12 +829,11 @@ def load_module(modname, args, module_type, user, network, retmsg, modpython):
         return 1
 
     module = cl()
-    module._cmod = CreatePyModule(user, network, modname, datapath, module, modpython)
+    module._cmod = CreatePyModule(user, network, modname, datapath, module_type, module, modpython)
     module.nv = ModuleNV(module._cmod)
     module.SetDescription(cl.description)
     module.SetArgs(args)
     module.SetModPath(pymodule.__file__)
-    module.SetType(module_type)
     _py_modules.add(module)
 
     if module_type == CModInfo.UserModule:
@@ -628,9 +918,12 @@ def unload_all():
         # add it back to set, otherwise unload_module will be sad
         _py_modules.add(mod)
         unload_module(mod)
+    if _cov:
+        _cov.stop()
 
 
 def gather_mod_info(cl, modinfo):
+    translation = CTranslationDomainRefHolder("znc-" + modinfo.GetName())
     modinfo.SetDescription(cl.description)
     modinfo.SetWikiPage(cl.wiki_page)
     modinfo.SetDefaultType(cl.module_types[0])
@@ -656,33 +949,6 @@ def get_mod_info(modname, retmsg, modinfo):
     return 2
 
 
-def get_mod_info_path(path, modname, modinfo):
-    try:
-        x = imp.find_module(modname, [path])
-    except ImportError:
-        return 0
-    # x == (<open file './modules/admin.so', mode 'rb' at 0x7fa2dc748d20>,
-    #       './modules/admin.so', ('.so', 'rb', 3))
-    # x == (<open file './modules/pythontest.py', mode 'U' at 0x7fa2dc748d20>,
-    #       './modules/pythontest.py', ('.py', 'U', 1))
-    if x[0] is None and x[2][2] != imp.PKG_DIRECTORY:
-        return 0
-    try:
-        pymodule = imp.load_module(modname, *x)
-    except ImportError:
-        return 0
-    finally:
-        if x[0]:
-            x[0].close()
-    if modname not in pymodule.__dict__:
-        return 0
-    cl = pymodule.__dict__[modname]
-    modinfo.SetName(modname)
-    modinfo.SetPath(pymodule.__file__)
-    gather_mod_info(cl, modinfo)
-    return 1
-
-
 CONTINUE = CModule.CONTINUE
 HALT = CModule.HALT
 HALTMODS = CModule.HALTMODS
@@ -709,6 +975,8 @@ def CreateWebSubPage(name, title='', params=dict(), admin=False):
 
 CUser.GetNetworks = CUser.GetNetworks_
 CIRCNetwork.GetChans = CIRCNetwork.GetChans_
+CIRCNetwork.GetServers = CIRCNetwork.GetServers_
+CIRCNetwork.GetQueries = CIRCNetwork.GetQueries_
 CChan.GetNicks = CChan.GetNicks_
 CZNC.GetUserMap = CZNC.GetUserMap_
 
@@ -742,7 +1010,7 @@ CModule.AddSocket = FreeOwnership(func=CModule.AddSocket)
 CModule.AddSubPage = FreeOwnership(func=CModule.AddSubPage)
 
 
-class ModulesIter(collections.Iterator):
+class ModulesIter(collections.abc.Iterator):
     def __init__(self, cmod):
         self._cmod = cmod
 
@@ -756,6 +1024,12 @@ class ModulesIter(collections.Iterator):
 CModules.__iter__ = lambda cmod: ModulesIter(CModulesIter(cmod))
 
 
+# e.g. msg.As(znc.CNumericMessage)
+def _CMessage_As(self, cl):
+    return getattr(self, 'As_' + cl.__name__, lambda: self)()
+CMessage.As = _CMessage_As
+
+
 def str_eq(self, other):
     if str(other) == str(self):
         return True
