@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2020 ZNC, see the NOTICE file for details.
+ * Copyright (C) 2004-2024 ZNC, see the NOTICE file for details.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -196,6 +196,7 @@ class CSASLMod : public CModule {
     }
 
     void Authenticate(const CString& sLine) {
+        if (m_Mechanisms.empty()) return;
         /* Send blank authenticate for other mechanisms (like EXTERNAL). */
         CString sAuthLine;
         if (m_Mechanisms.GetCurrent().Equals("PLAIN") && sLine.Equals("+")) {
@@ -249,6 +250,7 @@ class CSASLMod : public CModule {
     }
 
     EModRet OnNumericMessage(CNumericMessage& msg) override {
+        if (m_Mechanisms.empty()) return CONTINUE;
         if (msg.GetCode() == 903) {
             /* SASL success! */
             if (m_bVerbose) {
@@ -283,6 +285,8 @@ class CSASLMod : public CModule {
             m_bAuthenticated = true;
             GetNetwork()->GetIRCSock()->ResumeCap();
             DEBUG("sasl: Received 907 -- We are already registered");
+        } else if (msg.GetCode() == 908) {
+            return HALT;
         } else {
             return CONTINUE;
         }
