@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2020 ZNC, see the NOTICE file for details.
+ * Copyright (C) 2004-2024 ZNC, see the NOTICE file for details.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -453,6 +453,46 @@ void CClient::UserCommand(CString& sLine) {
             PutStatus(t_p("Disabled {1} channel", "Disabled {1} channels",
                           uDisabled)(uDisabled));
         }
+    } else if (sCommand.Equals("MOVECHAN")) {
+        if (!m_pNetwork) {
+            PutStatus(t_s(
+                "You must be connected with a network to use this command"));
+            return;
+        }
+
+        const auto sChan = sLine.Token(1);
+        const auto sTarget = sLine.Token(2);
+        if (sChan.empty() || sTarget.empty()) {
+            PutStatus(t_s("Usage: MoveChan <#chan> <index>"));
+            return;
+        }
+
+        unsigned int uIndex = sTarget.ToUInt();
+
+        CString sError;
+        if (m_pNetwork->MoveChan(sChan, uIndex - 1, sError))
+            PutStatus(t_f("Moved channel {1} to index {2}")(sChan, uIndex));
+        else
+            PutStatus(sError);
+    } else if (sCommand.Equals("SWAPCHANS")) {
+        if (!m_pNetwork) {
+            PutStatus(t_s(
+                "You must be connected with a network to use this command"));
+            return;
+        }
+
+        const auto sChan1 = sLine.Token(1);
+        const auto sChan2 = sLine.Token(2);
+        if (sChan1.empty() || sChan2.empty()) {
+            PutStatus(t_s("Usage: SwapChans <#chan1> <#chan2>"));
+            return;
+        }
+
+        CString sError;
+        if (m_pNetwork->SwapChans(sChan1, sChan2, sError))
+            PutStatus(t_f("Swapped channels {1} and {2}")(sChan1, sChan2));
+        else
+            PutStatus(sError);
     } else if (sCommand.Equals("LISTCHANS")) {
         if (!m_pNetwork) {
             PutStatus(t_s(
@@ -496,6 +536,7 @@ void CClient::UserCommand(CString& sLine) {
         }
 
         CTable Table;
+        Table.AddColumn(t_s("Index", "listchans"));
         Table.AddColumn(t_s("Name", "listchans"));
         Table.AddColumn(t_s("Status", "listchans"));
         Table.AddColumn(t_s("In config", "listchans"));
@@ -508,10 +549,12 @@ void CClient::UserCommand(CString& sLine) {
             Table.AddColumn(CString(cPerm));
         }
 
-        unsigned int uNumDetached = 0, uNumDisabled = 0, uNumJoined = 0;
+        unsigned int uNumDetached = 0, uNumDisabled = 0, uNumJoined = 0,
+                     uChanIndex = 1;
 
         for (const CChan* pChan : vChans) {
             Table.AddRow();
+            Table.SetCell(t_s("Index", "listchans"), CString(uChanIndex));
             Table.SetCell(t_s("Name", "listchans"),
                           pChan->GetPermStr() + pChan->GetName());
             Table.SetCell(
@@ -545,12 +588,19 @@ void CClient::UserCommand(CString& sLine) {
             if (pChan->IsDetached()) uNumDetached++;
             if (pChan->IsOn()) uNumJoined++;
             if (pChan->IsDisabled()) uNumDisabled++;
+
+            uChanIndex++;
         }
 
         PutStatus(Table);
         PutStatus(t_f("Total: {1}, Joined: {2}, Detached: {3}, Disabled: {4}")(
             vChans.size(), uNumJoined, uNumDetached, uNumDisabled));
     } else if (sCommand.Equals("ADDNETWORK")) {
+        if (!m_pUser->IsAdmin() && m_pUser->DenySetNetwork()) {
+            PutStatus(t_s("Access denied!"));
+            return;
+        }
+
         if (!m_pUser->IsAdmin() && !m_pUser->HasSpaceForNewNetwork()) {
             PutStatus(t_s(
                 "Network number limit reached. Ask an admin to increase the "
@@ -582,6 +632,11 @@ void CClient::UserCommand(CString& sLine) {
             PutStatus(sNetworkAddError);
         }
     } else if (sCommand.Equals("DELNETWORK")) {
+        if (!m_pUser->IsAdmin() && m_pUser->DenySetNetwork()) {
+            PutStatus(t_s("Access denied!"));
+            return;
+        }
+
         CString sNetwork = sLine.Token(1);
 
         if (sNetwork.empty()) {
@@ -757,6 +812,11 @@ void CClient::UserCommand(CString& sLine) {
             PutStatus(t_f("You don't have a network named {1}")(sNetwork));
         }
     } else if (sCommand.Equals("ADDSERVER")) {
+        if (!m_pUser->IsAdmin() && m_pUser->DenySetNetwork()) {
+            PutStatus(t_s("Access denied!"));
+            return;
+        }
+
         CString sServer = sLine.Token(1);
 
         if (!m_pNetwork) {
@@ -778,6 +838,11 @@ void CClient::UserCommand(CString& sLine) {
                     "added or openssl is disabled?"));
         }
     } else if (sCommand.Equals("REMSERVER") || sCommand.Equals("DELSERVER")) {
+        if (!m_pUser->IsAdmin() && m_pUser->DenySetNetwork()) {
+            PutStatus(t_s("Access denied!"));
+            return;
+        }
+
         if (!m_pNetwork) {
             PutStatus(t_s(
                 "You must be connected with a network to use this command"));
@@ -1781,6 +1846,11 @@ void CClient::HelpUser(const CString& sFilter) {
                    t_s("Enable channels", "helpcmd|EnableChan|desc"));
     AddCommandHelp("DisableChan", t_s("<#chans>", "helpcmd|DisableChan|args"),
                    t_s("Disable channels", "helpcmd|DisableChan|desc"));
+    AddCommandHelp("MoveChan", t_s("<#chan> <index>", "helpcmd|MoveChan|args"),
+                   t_s("Move channel in sort order", "helpcmd|MoveChan|desc"));
+    AddCommandHelp(
+        "SwapChans", t_s("<#chan1> <#chan2>", "helpcmd|SwapChans|args"),
+        t_s("Swap channels in sort order", "helpcmd|SwapChans|desc"));
     AddCommandHelp("Attach", t_s("<#chans>", "helpcmd|Attach|args"),
                    t_s("Attach to channels", "helpcmd|Attach|desc"));
     AddCommandHelp("Detach", t_s("<#chans>", "helpcmd|Detach|args"),
