@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2020 ZNC, see the NOTICE file for details.
+ * Copyright (C) 2004-2024 ZNC, see the NOTICE file for details.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -59,7 +59,7 @@ CZNC::CZNC()
       m_uiConnectDelay(5),
       m_uiAnonIPLimit(10),
       m_uiMaxBufferSize(500),
-      m_uDisabledSSLProtocols(Csock::EDP_SSL),
+      m_uDisabledSSLProtocols(Csock::EDP_SSL | Csock::EDP_TLSv1 | Csock::EDP_TLSv1_1),
       m_pModules(new CModules),
       m_uBytesRead(0),
       m_uBytesWritten(0),
@@ -134,15 +134,7 @@ CString CZNC::GetTag(bool bIncludeVersion, bool bHTML) {
 }
 
 CString CZNC::GetCompileOptionsString() {
-    // Build system doesn't affect ABI
-    return ZNC_COMPILE_OPTIONS_STRING + CString(
-                                            ", build: "
-#ifdef BUILD_WITH_CMAKE
-                                            "cmake"
-#else
-                                            "autoconf"
-#endif
-                                            );
+    return ZNC_COMPILE_OPTIONS_STRING;
 }
 
 CString CZNC::GetUptime() const {
@@ -742,10 +734,8 @@ bool CZNC::WriteNewConfig(const CString& sConfigFile) {
     } while (!CUser::IsValidUsername(sUser));
 
     vsLines.push_back("<User " + sUser + ">");
-    CString sSalt;
-    sAnswer = CUtils::GetSaltedHashPass(sSalt);
-    vsLines.push_back("\tPass       = " + CUtils::sDefaultHash + "#" + sAnswer +
-                      "#" + sSalt + "#");
+    sAnswer = CUtils::AskSaltedHashPassForConfig();
+	vsLines.push_back(sAnswer);
 
     vsLines.push_back("\tAdmin      = true");
 
@@ -786,7 +776,7 @@ bool CZNC::WriteNewConfig(const CString& sConfigFile) {
         CUtils::PrintMessage("");
 
         do {
-            CUtils::GetInput("Name", sNetwork, "freenode");
+            CUtils::GetInput("Name", sNetwork, "libera");
         } while (!CIRCNetwork::IsValidNetwork(sNetwork));
 
         vsLines.push_back("\t<Network " + sNetwork + ">");
@@ -803,8 +793,8 @@ bool CZNC::WriteNewConfig(const CString& sConfigFile) {
         bool bSSL = false;
         unsigned int uServerPort = 0;
 
-        if (sNetwork.Equals("freenode")) {
-            sHost = "chat.freenode.net";
+        if (sNetwork.Equals("libera")) {
+            sHost = "irc.libera.chat";
 #ifdef HAVE_LIBSSL
             bSSL = true;
 #endif
@@ -1060,6 +1050,7 @@ bool CZNC::ReadConfig(CConfig& config, CString& sError) {
     // create a backup file if necessary
     CString sSavedVersion;
     config.FindStringEntry("version", sSavedVersion);
+    config.AddKeyValuePair("version", sSavedVersion);
     if (sSavedVersion.empty()) {
         CUtils::PrintError(
             "Config does not contain a version identifier. It may be be too "
@@ -1102,20 +1093,28 @@ bool CZNC::RehashConfig(CString& sError) {
 bool CZNC::LoadGlobal(CConfig& config, CString& sError) {
     sError.clear();
 
+    CString sSavedVersion;
+    config.FindStringEntry("version", sSavedVersion);
+    tuple<unsigned int, unsigned int> tSavedVersion =
+        make_tuple(sSavedVersion.Token(0, false, ".").ToUInt(),
+                   sSavedVersion.Token(1, false, ".").ToUInt());
+
     MCString msModules;  // Modules are queued for later loading
 
     VCString vsList;
     config.FindStringVector("loadmodule", vsList);
+
+    // Automatically load corecaps if config was upgraded from old version, but
+    // don't force it if user explicitly unloaded it
+    if (tSavedVersion < make_tuple(1, 9)) {
+        vsList.push_back("corecaps");
+    }
+
     for (const CString& sModLine : vsList) {
         CString sModName = sModLine.Token(0);
         CString sArgs = sModLine.Token(1, true);
 
         // compatibility for pre-1.0 configs
-        CString sSavedVersion;
-        config.FindStringEntry("version", sSavedVersion);
-        tuple<unsigned int, unsigned int> tSavedVersion =
-            make_tuple(sSavedVersion.Token(0, false, ".").ToUInt(),
-                       sSavedVersion.Token(1, false, ".").ToUInt());
         if (sModName == "saslauth" && tSavedVersion < make_tuple(0, 207)) {
             CUtils::PrintMessage(
                 "saslauth module was renamed to cyrusauth. Loading cyrusauth "
