@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2020 ZNC, see the NOTICE file for details.
+ * Copyright (C) 2004-2024 ZNC, see the NOTICE file for details.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -22,6 +22,7 @@
 using ::testing::IsEmpty;
 using ::testing::ContainerEq;
 using ::testing::ElementsAre;
+using ::testing::SizeIs;
 
 TEST(MessageTest, SetParam) {
     CMessage msg;
@@ -609,3 +610,12 @@ TEST(MessageTest, ParseWithoutSourceAndTags) {
     EXPECT_EQ(msg.GetCommand(), "COMMAND");
     EXPECT_EQ(msg.GetParams(), VCString());
 }
+
+TEST(MessageTest, HugeParse) {
+    CString line;
+    for (int i = 0; i < 1000000; ++i) {
+        line += "a ";
+    }
+    CMessage msg(line);
+    EXPECT_THAT(msg.GetParams(), SizeIs(999999));
+}
