@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2020 ZNC, see the NOTICE file for details.
+ * Copyright (C) 2004-2024 ZNC, see the NOTICE file for details.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -37,7 +37,7 @@ class ZNCTest : public testing::Test {
 
     Socket ConnectIRCd();
     Socket ConnectClient();
-    Socket LoginClient();
+    Socket LoginClient(QString identifier = "");
 
     std::unique_ptr<Process> Run();
 
@@ -47,6 +47,7 @@ class ZNCTest : public testing::Test {
     std::unique_ptr<QNetworkReply> HandleHttp(QNetworkReply* reply);
 
     void InstallModule(QString name, QString content);
+    void InstallTranslation(QString module, QString language, QString content);
 
     App m_app;
     QNetworkAccessManager m_network;
